# Django
from django.utils.copycompat import deepcopy
from django.utils import simplejson

__all__ = ['hungarian_to_python', 'lookupattr']

def dumpjs(obj, *args, **kwargs):
    """Dump a Python object as Javascript, with support for a __json__ method."""
    class Encoder(simplejson.JSONEncoder):
        def iterencode(self, o, _one_shot=False):
            #print o
            if hasattr(o, '__json__'):
                if callable(o.__json__):
                    print 'foo', o
                    return o.__json__()
                else:
                    return o.__json__
            else:
                return super(Encoder, self).iterencode(o)#, _one_shot=_one_shot)
    kwargs['cls'] = Encoder
    kwargs['sort_keys'] = True
    #return simplejson.dumps(obj, *args, **kwargs)
    output = simplejson.dumps(obj, *args, **kwargs)
    for key, val in obj.items():
        if 'fn' == key[0:2]:
            output = output.replace(simplejson.dumps(val), val)
    return output

class fn(object):
    """Wrapper for a Javascript function that should be encoded without escaping."""
    def __init__(self, fndef):
        self.fndef = unicode(fndef)
    def __getattr__(self, name):
        return getattr(self.fndef, name)
    def __repr__(self):
        return 'fn(%r)' % self.fndef
    def __unicode__(self):
        return self.fndef
    def __json__(self):
        return unicode(self.fndef)
    def __deepcopy__(self, memo):
        return deepcopy(self.fndef, memo)
        #return self.__class__(deepcopy(self.fndef, memo))

def hungarian_to_python(name, value):
    """Validate DataTable options specified in Hungarian notation."""
    if value is None:
        return value
    elif name.startswith('fn') and name[2].isupper():
        return fn(value)
    elif name.startswith('n') and name[1].isupper():
        return value
    elif name.startswith('m') and name[1].isupper():
        return value
    elif name.startswith('o') and name[1].isupper():
        d = {}
        for k, v in dict(value).iteritems():
            d[k] = hungarian_to_python(k, v)
        return d
    elif name.startswith('a') and name[1].isupper():
        return list(value)
    elif name.startswith('a') and name[1] in 'abfimnos' and name[2].isupper():
        a = []
        for i in list(value):
            a.append(hungarian_to_python(name[1:], i))
        return a
    elif name.startswith('s') and name[1].isupper():
        return unicode(value)
    elif name.startswith('b') and name[1].isupper():
        return bool(str(value).lower() in ('t', 'true', 'yes', 'y', '1'))
    elif name.startswith('f') and name[1].isupper():
        return float(value)
    elif name.startswith('i') and name[1].isupper():
        return int(value)
    else:
        raise NameError, 'name "%s" is not in hungarian notation' % name

def lookupattr(obj, name, default=None):
    """Recursively lookup an attribute or key on an object."""
    name = name.replace('__', '.')
    for element in name.split('.'):
        try:
            attr = obj.__getattribute__(element)
        except AttributeError:
            try:
                attr = obj.__dict__[element]
            except (KeyError, AttributeError):
                try:
                    attr = obj[element]
                except (KeyError, TypeError):
                    attr = default
                    break
        except:
            attr = default
            break
        if callable(attr):
            obj = attr()
        else:
            obj = attr
    if callable(attr):
        return attr()
    else:
        return attr



# Python
import os

# django-setuptest
import setuptest

class TestSuite(setuptest.SetupTestSuite):
    
    def resolve_packages(self):
        packages = super(TestSuite, self).resolve_packages()
        test_app = os.path.basename(os.path.dirname(__file__))
        if test_app not in packages:
            packages.append(test_app)
        return packages



# Python
import os
import sys

# Django
from django.conf import global_settings

# Update this module's local settings from the global settings module.
this_module = sys.modules[__name__]
for setting in dir(global_settings):
    if setting == setting.upper():
        setattr(this_module, setting, getattr(global_settings, setting))

# Absolute path to the directory containing this Django project.
PROJECT_ROOT = os.path.abspath(os.path.dirname(__file__))

DEBUG = True
TEMPLATE_DEBUG = DEBUG

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(PROJECT_ROOT, 'test_project.sqlite3'),
    }
}

SITE_ID = 1

STATIC_URL = '/static/'

MIDDLEWARE_CLASSES += (
    'debug_toolbar.middleware.DebugToolbarMiddleware',
)

TEMPLATE_DIRS = (
    #os.path.join(PROJECT_ROOT, 'templates'),
)

ROOT_URLCONF = 'test_project.urls'

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    'debug_toolbar',
    'south',
    'sortedm2m',
    'fortunecookie',
    'datatables',
    'test_app',
)

INTERNAL_IPS = ('127.0.0.1',)

DEBUG_TOOLBAR_CONFIG = {
    'INTERCEPT_REDIRECTS': False,
}



#!/usr/bin/env python

# Python
import imp
import os
import sys

# Django
from django.core.management import execute_manager

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
import settings

if __name__ == "__main__":
    execute_manager(settings)



# Decorator
from decorator import decorator

__all__ = ['datatable']

def datatable(datatable_class, name='datatable'):
    def datatable(f, request, *args, **kwargs):
        datatable_instance = datatable_class(request.GET, name)
        response = datatable_instance.process_request(request, name)
        if response is not None:
            return response
        response = f(request, *args, **kwargs)
        return datatable_instance.process_response(request, response)
    return decorator(datatable)



# Django
from django.template.response import TemplateResponse

# Django-DataTables
import datatables

# Django-FortuneCookie
from fortunecookie.models import *

class FortuneCookieTable(datatables.DataTable):

    #pk = datatables.CheckboxColumn()
    fortune = datatables.Column(
        label='Your Fortune',
        sort_field='fortune_lower',
        )

    fortune_lower = datatables.Column(
        label='Lower Fortune',
        bVisible=False,
        bSearchable=False,
        )

    lucky_numbers = datatables.Column(
        display_field='lucky_numbers_display',
        sClass='okie',
        asSorting=["asc", "desc", "desc"],
        bSearchable=False,
        )

    chinese_word = datatables.Column(
        label='Chinese Word',
        sort_field='chinese_word.english_word',
        search_field='chinese_word.english_word',
        sClass='okie',
        #bSearchable=False,
        )

    def get_queryset(self):
        qs = self.base_queryset()
        qs = qs.extra(select={
            'fortune_lower': 'LOWER(fortune)',
        })
        return qs

    class Meta:
        model = FortuneCookie
        bInfo = True
        bSort = True
        bPaginate = False
        bJQueryUI = True
        sScrollY = '200px'
        aaSorting = [[2, "desc"]]
        aLengthMenu = [[3, 10, 25, 50, -1], [3, 10, 25, 50, "All"]]
        #fnInitComplete = 'function(oSettings, json) { alert("Init Complete!"); }'
        bServerSide = True
        sAjaxSource = '/'
        #fnServerData = 'function( sUrl, aoData, fnCallback ) {$.ajax( {"url": sUrl, "data": aoData, "success": fnCallback, "dataType": "jsonp", "cache": false} );}'

@datatables.datatable(FortuneCookieTable, name='fct')
def index(request):
    qs = request.fct.get_queryset()
    request.fct.update_queryset(qs)
    return TemplateResponse(request, 'index.html', {'table': request.fct})



# Python
import json

# Django
from django.test import TestCase
from django.utils.datastructures import SortedDict

# Django-DataTables
import datatables

class TestDataTables(TestCase):
    """Test cases for Django-DataTables app."""

    fixtures = ['fortunecookies']

    def test_hungarian_to_python(self):
        fndef = 'function(oSettings, json) { alert("Init Complete!"); }'
        result = datatables.hungarian_to_python('fnInitComplete', fndef)
        self.assertEqual(unicode(result), fndef)
        #self.assertEqual(datatables.utils.dumpjs(result), fndef)
        result = datatables.hungarian_to_python('bVisible', 1)
        self.assertTrue(isinstance(result, bool))
        self.assertTrue(result)

    def test_lookupattr(self):
        pass

    def _test_meta_options(self, **kwargs):
        dumps = json.dumps
        class DT(datatables.DataTable):
            Meta = type('Meta', (object,), kwargs)
        js_options = DT().js_options()
        for name, value in kwargs.items():
            expected_js = value
            if 'fn' != name[0:2]:
                expected_js = dumps(name) + ': ' + dumps(value, sort_keys=True)
            self.assertTrue(expected_js in js_options)

    def test_features(self):
        # Test that all DataTables features are handled by the Meta class and
        # included in the JS/JSON output from the DataTable class.
        # http://datatables.net/usage/features
        self._test_meta_options(bAutoWidth=True)
        self._test_meta_options(bDeferRender=True)
        self._test_meta_options(bFilter=True)
        self._test_meta_options(bInfo=True)
        self._test_meta_options(bJQueryUI=True)
        self._test_meta_options(bLengthChange=True)
        self._test_meta_options(bPaginate=True)
        self._test_meta_options(bProcessing=True)
        self._test_meta_options(bScrollInfinite=True)
        self._test_meta_options(bSort=True)
        self._test_meta_options(bSortClasses=True)
        self._test_meta_options(bStateSave=True)
        self._test_meta_options(sScrollX='100%')
        self._test_meta_options(sScrollY='100%')

    def test_options(self):
        # Test that all DataTables options are handled by the Meta class and
        # included in the JS/JSON output from the DataTable class.
        # http://datatables.net/usage/options
        aaData = [
            ['Trident', 'Internet Explorer 4.0', 'Win 95+', 4, 'X'],
            ['Trident', 'Internet Explorer 5.0', 'Win 95+', 5, 'C'],
        ]
        self._test_meta_options(aaData=aaData)
        self._test_meta_options(aaSorting=[[2, 'asc'], [3, 'desc']])
        self._test_meta_options(aaSortingFixed=[[0, 'asc']])
        aLengthMenu = [[10, 25, 50, -1], [10, 25, 50, "All"]]
        self._test_meta_options(aLengthMenu=aLengthMenu)
        aoSearchCols = [
            None,
            {'sSearch': 'My filter'},
            None,
            {'sSearch': '^[0-9]', 'bEscapeRegex': False}
        ]
        self._test_meta_options(aoSearchCols=aoSearchCols)
        asStripClasses = ['strip1', 'strip2', 'strip3']
        self._test_meta_options(asStripClasses=asStripClasses)
        self._test_meta_options(bDestroy=True)
        self._test_meta_options(bRetrieve=True)
        self._test_meta_options(bScrollCollapse=True)
        self._test_meta_options(bSortCellsTop=True)
        self._test_meta_options(iCookieDuration=60*60*24)
        self._test_meta_options(iDeferLoading=57)
        self._test_meta_options(iDisplayLength=50)
        self._test_meta_options(iDisplayStart=20)
        self._test_meta_options(iScrollLoadGap=50)
        self._test_meta_options(oSearch={'sSearch': 'Initial search'})
        self._test_meta_options(sAjaxDataProp='data')
        sAjaxSource = 'http://www.sprymedia.co.uk/dataTables/json.php'
        self._test_meta_options(sAjaxSource=sAjaxSource)
        self._test_meta_options(sCookiePrefix='my_datatable_')
        self._test_meta_options(sDom='<"top"i>rt<"bottom"flp><"clear"&lgt;')
        self._test_meta_options(sPaginationType='full_numbers')
        self._test_meta_options(sScrollXInner='110%')

    def test_callbacks(self):
        # Test that all DataTables callbacks are handled by the Meta class and
        # included in the JS/JSON output from the DataTable class.
        # http://datatables.net/usage/callbacks
        fnCookieCallback = '''function (sName, oData, sExpires, sPath) {
            /* Customise oData or sName or whatever else here */
            return sName + "="+JSON.stringify(oData)+"; expires=" + sExpires +"; path=" + sPath;
        }'''
        self._test_meta_options(fnCookieCallback=fnCookieCallback)
        fnDrawCallback = '''function(oSettings) {
            alert('DataTables has redrawn the table');
        }'''
        self._test_meta_options(fnDrawCallback=fnDrawCallback)
        fnFooterCallback = '''function(nFoot, aasData, iStart, iEnd, aiDisplay) {
            nFoot.getElementsByTagName('th')[0].innerHTML = "Starting index is "+iStart;
        }'''
        self._test_meta_options(fnFooterCallback=fnFooterCallback)
        fnFormatNumber = '''function (iIn) {
            return iIn;
        }'''
        self._test_meta_options(fnFormatNumber=fnFormatNumber)
        fnHeaderCallback = '''function(nHead, aasData, iStart, iEnd, aiDisplay) {
            nHead.getElementsByTagName('th')[0].innerHTML = "Displaying "+(iEnd-iStart)+" records";
        }'''
        self._test_meta_options(fnHeaderCallback=fnHeaderCallback)
        fnInfoCallback = '''function(oSettings, iStart, iEnd, iMax, iTotal, sPre) {
            return iStart +" to "+ iEnd;
        }'''
        self._test_meta_options(fnInfoCallback=fnInfoCallback)
        fnInitComplete = '''function(oSettings, json) {
            alert('DataTables has finished its initialisation.');
        }'''
        self._test_meta_options(fnInitComplete=fnInitComplete)
        fnPreDrawCallback = '''function(oSettings) {
            if ($('#test').val() == 1) {
                return false;
            }
        }'''
        self._test_meta_options(fnPreDrawCallback=fnPreDrawCallback)
        fnRowCallback = '''function(nRow, aData, iDisplayIndex, iDisplayIndexFull) {
            /* Bold the grade for all 'A' grade browsers */
            if ( aData[4] == "A" )
            {
                $('td:eq(4)', nRow).html( '<b>A</b>' );
            }
            return nRow;
        }'''
        self._test_meta_options(fnRowCallback=fnRowCallback)
        fnServerData = '''function (sSource, aoData, fnCallback) {
            /* Add some data to send to the source, and send as 'POST' */
            aoData.push({ "name": "my_field", "value": "my_value" });
            $.ajax({
                "dataType": 'json', 
                "type": "POST", 
                "url": sSource, 
                "data": aoData, 
                "success": fnCallback
            });
        }'''
        self._test_meta_options(fnServerData=fnServerData)
        fnServerParams = '''function (aoData) {
            aoData.push({"name": "more_data", "value": "my_value"});
        }'''
        self._test_meta_options(fnServerParams=fnServerParams)
        fnStateLoadCallback = '''function (oSettings, oData) {
            oData.sFilter = "";
            return true;
        }'''
        self._test_meta_options(fnStateLoadCallback=fnStateLoadCallback)
        fnStateSaveCallback = '''function (oSettings, sValue) {
            sValue += ',"myCustomParameter": "myValue"';
            return sValue;
        }'''
        self._test_meta_options(fnStateSaveCallback=fnStateSaveCallback)

    def _test_column_options(self, **kwargs):
        dumps = json.dumps
        columns = SortedDict()
        option_values = {}
        for index, name in enumerate(kwargs.keys()):
            options = kwargs[name]
            columns[name] = datatables.Column(**options)
            for k,v in options.items():
                if k not in option_values:
                    option_values[k] = []
                if v not in [x[0] for x in option_values[k]]:
                    option_values[k].append([v, []])
                for x in option_values[k]:
                    if x[0] == v:
                        x[1].append(index)
        DT = type('DT', (datatables.DataTable,), columns)
        js_options = DT().js_options()
        for oname, ovalues in option_values.items():
            for ovalue, otargets in ovalues:
                expected_js = dumps({
                    'aTargets': otargets,
                    oname: ovalue,
                }, sort_keys=True).lstrip('{').rstrip('}').strip()
            #print js_options, expected_js
            self.assertTrue(expected_js in js_options)

    def test_columns(self):
        # Test that all DataTables column options are handled by the Column
        # class and included in the JS/JSON output from the DataTable class.
        # http://datatables.net/usage/columns
        options = {
            'fortune': {'asSorting': ['desc', 'asc', 'asc']},
            'lucky_numbers': {},
        }
        # FIXME: self._test_column_options(**options)
        options = {
            'fortune': {'bSearchable': False},
            'lucky_numbers': {},
        }
        self._test_column_options(**options)
        options = {
            'fortune': {'bSortable': False},
            'lucky_numbers': {'bSortable': False},
        }
        self._test_column_options(**options)
        options = {
            'fortune': {'bUseRendered': False},
            'lucky_numbers': {},
        }
        self._test_column_options(**options)
        options = {
            'fortune': {'bVisible': False},
            'lucky_numbers': {'bVisible': False},
        }
        self._test_column_options(**options)
        options = {
            'fortune': {'fnRender': 'function (oObj) { return oObj.aData[oObj.iDataColumn]; }'},
            'lucky_numbers': {},
        }
        # FIXME: self._test_column_options(**options)
        options = {
            'fortune': {'mDataProp': 'fortune_display'},
            'lucky_numbers': {'mDataProp': 1},
        }
        self._test_column_options(**options)
        options = {
            'fortune': {'mDataProp': 'function(oObj) { return oObj.aData[0]; }'},
            'lucky_numbers': {'mDataProp': None},
        }
        # FIXME: self._test_column_options(**options)
        options = {
            'fortune': {'sClass': 'my_class'},
            'lucky_numbers': {'sClass': 'my_class'},
        }
        self._test_column_options(**options)
        options = {
            'fortune': {},
            'lucky_numbers': {'sDefaultContent': 'Edit'},
        }
        self._test_column_options(**options)
        options = {
            'fortune': {},
            'lucky_numbers': {'sName': 'lucky_number_column'},
        }
        self._test_column_options(**options)
        options = {
            'fortune': {'sSortDataType': 'dom-text'},
            'lucky_numbers': {'sSortDataType': 'dom-text'},
        }
        self._test_column_options(**options)
        options = {
            'fortune': {'sTitle': 'My column title'},
            'lucky_numbers': {},
        }
        self._test_column_options(**options)
        options = {
            'fortune': {'sType': 'html'},
            'lucky_numbers': {'sType': 'html'},
        }
        self._test_column_options(**options)
        options = {
            'fortune': {'sWidth': '20%'},
            'lucky_numbers': {},
        }
        self._test_column_options(**options)

    def test_column_sorting(self):
        class DT(datatables.DataTable):
            fortune = datatables.Column(sort_field="lucky_numbers")
            lucky_numbers = datatables.Column()

        dt = DT()
        js_options = dt.js_options()
        self.assertTrue('"aTargets": [0], "iDataSort": 1' in js_options)



#!/usr/bin/env python

# Setuptools
from setuptools import setup, find_packages

# Read version from datatables/__init__.py
__VERSION__ = [line for line in file('datatables/__init__.py', 'rb') \
               if line.startswith('__VERSION__')][0].split(\
               '=')[1].strip().lstrip('\'').rstrip('\'')

setup(
    name='django-datatables',
    version=__VERSION__,
    author='Nine More Minutes, Inc.',
    author_email='support@ninemoreminutes.com',
    description='Django integration of DataTables jQuery plugin.',
    long_description=file('README', 'rb').read(),
    license='BSD',
    keywords='django datatables jquery tables',
    url='https://projects.ninemoreminutes.com/projects/django-datatables/',
    packages=find_packages(exclude=['test_app', 'test_project']),
    include_package_data=True,
    zip_safe=False,
    install_requires=['Django>=1.3', 'decorator'],
    setup_requires=[],
    tests_require=['Django>=1.3', 'decorator', 'django-fortunecookie>=0.1.1', 'django-setuptest'],
    test_suite='test_app.TestSuite',
    classifiers=[
        'Development Status :: 1 - Planning',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries',
    ],
    options={
        'egg_info': {
            'tag_svn_revision': 1,
            'tag_build': '.dev',
        },
        'build_sphinx': {
            'source_dir': 'docs',
            'build_dir': 'docs/_build',
            'all_files': True,
            'version': __VERSION__,
            'release': __VERSION__,
        },
        'upload_sphinx': {
            'upload_dir': 'docs/_build/html',
        },
        'aliases': {
            # FIXME: Add test to aliases below.
            'dev_build': 'egg_info sdist build_sphinx',
            'release_build': 'egg_info -b "" sdist build_sphinx',
            'release_and_upload': 'egg_info -b "" sdist register upload',
        },
    },
)



# Django
from django.http import HttpResponse
from django.template import Context
from django.template.loader import select_template
from django.utils.copycompat import deepcopy
from django.utils.datastructures import SortedDict
from django.utils.safestring import mark_safe
from django.db.models import Q
from django.forms.widgets import Media, media_property

# Django DataTables
from columns import Column, BoundColumn
from utils import dumpjs, hungarian_to_python, lookupattr

__all__ = ['DataTable']

class DataTableOptions(object):
    """Container class for DataTable options defined via the Meta class."""

    def __init__(self, options=None):
        self.update(options)

    def update(self, options=None):
        self.id = getattr(options, 'id', getattr(self, 'id', 'datatable_%d' % id(self)))
        self.var = getattr(options, 'var', getattr(self, 'var', None))
        self.classes = getattr(options, 'classes', getattr(self, 'classes', []))
        if isinstance(self.classes, basestring):
            self.classes = self.classes.split()
        self.classes = set(self.classes)
        self.width = str(getattr(options, 'width', getattr(self, 'width', '100%')))
        self.border = str(getattr(options, 'border', getattr(self, 'border', '0')))
        self.cellpadding = str(getattr(options, 'cellpadding', getattr(self, 'cellpadding', '0')))
        self.cellspacing = str(getattr(options, 'cellspacing', getattr(self, 'cellspacing', '0')))
        self.model = getattr(options, 'model', getattr(self, 'model', None))
        self.options = getattr(self, 'options', {})
        self.options.update(getattr(options, 'options', {}))
        for name in dir(options):
            if name.startswith('_'):
                continue
            value = getattr(options, name)
            try:
                self.options[name] = hungarian_to_python(name, value)
            except NameError:
                pass

class DataTableDeclarativeMeta(type):
    """Metaclass for capturing declarative attributes on a DataTable class."""

    def __new__(cls, name, bases, attrs):
        columns = [(c, attrs.pop(c)) for c, obj in attrs.items() \
                  if isinstance(obj, Column)]
        columns.sort(key=lambda x: x[1].creation_counter)
        for base in reversed(bases):
            columns = getattr(base, 'base_columns', {}).items() + columns
        attrs['base_columns'] = SortedDict(columns)
        new_class = super(DataTableDeclarativeMeta, cls).__new__(cls, name, bases, attrs)
        new_class._meta = DataTableOptions()
        for base in reversed(bases):
            new_class._meta.update(getattr(base, '_meta', None))
        new_class._meta.update(getattr(new_class, 'Meta', None))
        if 'media' not in attrs:
            new_class.media = media_property(new_class)
        return new_class

class DataTable(object):
    """Base class for defining a DataTable and options."""

    __metaclass__ = DataTableDeclarativeMeta

    def __init__(self, data=None, name=''):
        self.columns = SortedDict(self.base_columns.items())#deepcopy(self.base_columns)

    @property
    def id(self):
        return self._meta.id

    @property
    def var(self):
        return self._meta.var

    @property
    def classes(self):
        return ' '.join(self._meta.classes)

    @property
    def width(self):
        return self._meta.width

    @property
    def border(self):
        return self._meta.border

    @property
    def cellpadding(self):
        return self._meta.cellpadding

    @property
    def cellspacing(self):
        return self._meta.cellspacing

    @property
    def bound_columns(self):
        if not getattr(self, '_bound_columns', None):
            self._bound_columns = SortedDict([
                (name, BoundColumn(self, column, name)) \
                for name, column in self.columns.items()])
        return self._bound_columns

    def get_default_queryset(self):
        """Return the default queryset to be used for this table."""
        if self._meta.model:
            return self._meta.model._default_manager.get_query_set()
        else:
            return EmptyQuerySet()

    def get_queryset(self):
        """Return the current queryset to use for this table."""
        return getattr(self, '_qs', self.get_default_queryset())

    def update_queryset(self, qs):
        """Update the queryset used for this table."""
        self._qs = qs

    def reset_queryset(self):
        """Reset the queryset to the default for this table."""
        if hasattr(self, '_qs'):
            del self._qs

    def rows(self):
        if self._meta.options.get('bServerSide', False):
            qs = self.get_queryset().none()
        else:
            qs = self.get_queryset()
        for result in qs:
            d = SortedDict()
            for bcol in self.bound_columns.values():
                d[bcol.name] = bcol.render_value(result)
            if 'id' not in d:
                d['id'] = result.id
            yield d

    def js_options(self):
        options = deepcopy(self._meta.options)
        columns = self.bound_columns
        aoColumnDefs = options.setdefault('aoColumnDefs', [])
        colopts = SortedDict()
        for index, name in enumerate(columns.keys()):
            column = columns[name]
            for key, value in column.options.items():
                if not (key, str(value)) in colopts.keys():
                    colopts[(key, str(value))] = {}
                    colopts[(key, str(value))]['targets'] = []
                colopts[(key, str(value))]['targets'] = colopts[(key, str(value))]['targets'] + [index]
                colopts[(key, str(value))]['key'] = key
                colopts[(key, str(value))]['value'] = value
            if column.sort_field != column.display_field and column.sort_field in columns:
                key = 'iDataSort'
                value = columns.keys().index(column.sort_field)
                if not (key, str(value)) in colopts.keys():
                    colopts[(key, str(value))] = {}
                    colopts[(key, str(value))]['targets'] = []
                colopts[(key, str(value))]['targets'] = colopts[(key, str(value))]['targets'] + [index]
                colopts[(key, str(value))]['key'] = key
                colopts[(key, str(value))]['value'] = value
        for kv, values in colopts.items():
            aoColumnDefs.append(dict([(values['key'], values['value']), ('aTargets', values['targets'])]))
        return mark_safe(dumpjs(options, indent=4, sort_keys=True))

    @property
    def media(self):
        media = Media()
        for bound_column in self.bound_columns.values():
            media = media + bound_column.column.media
        return media

    def extra_js(self):
        columns = self.bound_columns.values()
        extra = ''
        for column in columns:
            try:
                extra += column.column.render_javascript(self.var, column)
            except Exception, e:
                #print e
                pass
        return mark_safe(extra)

    def process_request(self, request, name='datatable'):
        setattr(request, name, self)
        
    def process_response(self, request, response):
        if 'sEcho' in request.GET:# and request.is_ajax():
            return self.handle_ajax(request)
        else:
            return response

    def _handle_ajax_sorting(self, qs, params):
        bc = self.bound_columns
        sort_fields = []
        iSortingCols = params.get('iSortingCols', 0)
        for i in range(iSortingCols):
            iSortCol = params.get('iSortCol_%d' % i, 0)
            bcol = bc.values()[iSortCol]
            sSortDir = params.get('sSortDir_%d' % i, '')
            bSortable = params.get('bSortable_%d' % i, True)
            bSortable = bSortable and bcol.options.get('bSortable', True)
            if not bSortable:
                continue
            sort_field = bcol.sort_field
            if sSortDir == 'desc':
                sort_field = '-%s' % sort_field
            sort_fields.append(sort_field)
        if sort_fields:
            qs = qs.order_by(*sort_fields)
        return qs

    def _handle_ajax_global_search(self, qs, params):
        bc = self.bound_columns
        search_fields = []
        sSearch = params.get('sSearch', '')
        bRegex = params.get('bRegex', False)
        if sSearch:
            iColumns = params.get('iColumns', 0)
            for i in range(iColumns):
                bcol = bc.values()[i]
                bSearchable = params.get('bSearchable_%d' % i, True)
                bSearchable = bSearchable and bcol.options.get('bSearchable', True)
                if bSearchable:
                    search_fields.append(bcol.search_field)
            qfilter = None
            for search_field in search_fields:
                # FIXME: Does not work for extra fields or foreignkey fields
                if bRegex:
                    q = Q(**{'%s__regex' % search_field: sSearch})
                else:
                    q = Q(**{'%s__icontains' % search_field: sSearch})
                if qfilter is None:
                    qfilter = q 
                else:
                    qfilter |= q
            if qfilter:
                qs = qs.filter(qfilter)
        return qs

    def _handle_ajax_column_specific_search(self, qs, params):
        bc = self.bound_columns
        search_fields = []
        iColumns = params.get('iColumns', 0)
        for i in range(iColumns):
            bcol = bc.values()[i]
            bSearchable = params.get('bSearchable_%d' % i, True)
            bSearchable = bSearchable and bcol.options.get('bSearchable', True)
            sSearch = params.get('sSearch_%d' % i, '')
            bRegex = params.get('bRegex_%d' % i, False)
            if bSearchable and sSearch:
                search_fields.append({
                    'field': bcol.search_field,
                    'term': sSearch,
                    'regex': bRegex,
                })
        qfilter = None
        for search_field in search_fields:
            if search_field['regex']:
                q = Q(**{'%s__regex' % search_field['field']: search_field['term']})
            else:
                q = Q(**{'%s__icontains' % search_field['field']: search_field['term']})
            if qfilter is None:
                qfilter = q 
            else:
                qfilter |= q
        if qfilter:
            qs = qs.filter(qfilter)
        return qs

    def handle_ajax(self, request):
        params = {}
        for name, value in request.GET.items():
            try:
                params[name] = hungarian_to_python(name, value)
            except NameError:
                pass
        qs = self.get_queryset()
        iTotalRecords = qs.count()
        qs = self._handle_ajax_sorting(qs, params)
        qs = self._handle_ajax_global_search(qs, params)
        qs = self._handle_ajax_column_specific_search(qs, params)
        iTotalDisplayRecords = qs.count()
        iDisplayStart = params.get('iDisplayStart', 0)
        iDisplayLength = params.get('iDisplayLength', -1)
        if iDisplayLength < 0:
            iDisplayLength = iTotalDisplayRecords
        qs = qs[iDisplayStart:(iDisplayStart + iDisplayLength)]
        aaData = []
        for result in qs:
            aData = []
            for bcol in self.bound_columns.values():
                aData.append(bcol.render_value(result, include_hidden=False))
            aaData.append(aData)
        data = {
            'iTotalRecords': iTotalRecords,
            'iTotalDisplayRecords': iTotalDisplayRecords,
            'sEcho': params.get('sEcho', ''),
            #'sColumns': ,
            'aaData': aaData,
        }
        #print qs.query
        s = dumpjs(data)
        return HttpResponse(s, content_type='application/json')

    def as_html(self):
        t = select_template(['datatables/table.html'])
        return t.render(Context({'table': self}))



# Django
from django.utils.copycompat import deepcopy
from django.utils.safestring import mark_safe
from django.forms.widgets import media_property
from django.template import Context
from django.template.loader import select_template

# Django-DataTables
from utils import hungarian_to_python, lookupattr

__all__ = ['Column', 'CheckboxColumn', 'ExpandableColumn']

class ColumnMeta(type):
    """Metaclass for Column class and subclass creation."""

    def __new__(cls, name, bases, attrs):
        new_class = super(ColumnMeta, cls).__new__(cls, name, bases, attrs)
        if 'media' not in attrs:
            new_class.media = media_property(new_class)
        return new_class

class Column(object):
    """Specify options for a Column on a DataTable."""

    __metaclass__ = ColumnMeta

    creation_counter = 0

    DEFAULTS = {
        'id': None,
        'classes': None,
        'label': None,
        'model_field': None,
        'display_field': None,
        'sort_field': None,
        'search_field': None,
        'value_renderer': None,
        'label_renderer': None,
    }

    def __init__(self, **kwargs):
        self.options = {}
        for key, value in kwargs.items():
            try:
                self.options[key] = hungarian_to_python(key, value)
                kwargs.pop(key)
            except NameError:
                pass
        for key, value in self.DEFAULTS.items():
            setattr(self, key, kwargs.get(key, value))
        self.classes = set(self.classes or [])
        if self.value_renderer is None:
            self.value_renderer = getattr(self, 'render_value', None)
        if self.label_renderer is None:
            self.label_renderer = getattr(self, 'render_label', None)
        # Increase the creation counter, and save our local copy.
        self.creation_counter = Column.creation_counter
        Column.creation_counter += 1

class CheckboxColumn(Column):

    class Media:
        js = ('datatables/checkbox_column.js',)
        css = {'all': ('datatables/checkbox_column.css',)}

    def __init__(self, **kwargs):
        kwargs.setdefault('bSortable', False)
        self.name = kwargs.get('name', None)
        super(CheckboxColumn, self).__init__(**kwargs)
        self.classes.add('datatables_checkbox_column')
        self.template = select_template(['datatables/checkbox_column.html'])

    def render_label(self, bc):
        c = Context({
            'classes': ' '.join(self.classes),
            'name': self.name or bc.name,
            'value': '__ALL__',
        })
        #return mark_safe(self.template.render(c))
        return mark_safe(select_template(['datatables/checkbox_column_label.html']).render(c))

    def render_value(self, row, bc):
        c = Context({
            'classes': ' '.join(self.classes),
            'name': self.name or bc.name,
            'value': getattr(row, 'id', ''),
            'checked': bool(bc.model_field and lookupattr(row, bc.model_field)),
        })
        return mark_safe(self.template.render(c))

class ExpandableColumn(Column):

    #class Media:
    #    js = ('datatables/expandablecolumn.js',)

    def __init__(self, **kwargs):
        self.close_image = kwargs.pop('close_image', None)
        self.open_image = kwargs.pop('open_image', None)
        kwargs.setdefault('bSortable', False)
        self.function = kwargs.pop('function', None)
        super(ExpandableColumn, self).__init__(**kwargs)

    def render_label(self, bound_column):
        return ''#mark_safe(u'<img class="datatables_expand" src="%s" />' % (self.open_image))

    def render_value(self, row, bound_column):
        return mark_safe('<img class="datatables_expand" src="%s" />' % (self.open_image))

    def render_javascript(self, var, bound_column):
        javascript = '''
$(document).ready(function() {
  $('#%(id)s img.datatables_expand').live( 'click', function () {
    var nTr = $(this).parents('tr')[0];
    if ( %(var)s.fnIsOpen(nTr) ) {
      /* This row is already open - close it */
      this.src = '%(open_image)s';
      %(var)s.fnClose( nTr );
    } else {
      /* Open this row */
      this.src = '%(close_image)s';
      %(var)s.fnOpen( nTr, %(function)s, 'details' );
    }
  });
});
        ''' % {'var': var, 'open_image': self.open_image, 'close_image': self.close_image,
               'function': self.function, 'id': bound_column.datatable.id}
        return javascript

class BoundColumn(object):
    """A Column bound to a particular DataTable instance."""

    def __init__(self, datatable, column, name):
        self.datatable = datatable
        self.column = column
        self.name = name
        self.options = deepcopy(self.column.options)
        for key in self.column.DEFAULTS.keys():
            setattr(self, key, getattr(self.column, key))
        if self.id is None:
            self.id = '%s-%s' % (self.datatable.id, self.name)
        self.classes = set(self.classes)
        self.classes.add('datatable_col_%s' % self.name)
        if self.label is None:
            self.label = self.name.replace('_', ' ').title()
        if self.model_field is None:
            self.model_field = self.name
        self.model_field = self.model_field.replace('.', '__')
        if self.display_field is None:
            self.display_field = self.model_field
        self.display_field = self.display_field.replace('.', '__')
        if self.sort_field is None:
            self.sort_field = self.model_field
        self.sort_field = self.sort_field.replace('.', '__')
        if self.search_field is None:
            self.search_field = self.model_field
        self.search_field = self.search_field.replace('.', '__')

    @property
    def class_value(self):
        return ' '.join(self.classes)

    def render_label(self):
        if self.label_renderer:
            try:
                return self.label_renderer(self)
            except Exception, e:
                print e
                return self.label
        else:
            return self.label

    def render_value(self, row, include_hidden=True):
        if not include_hidden and not self.options.bVisible:
            return u''
        elif self.value_renderer:
            try:
                value = self.value_renderer(row, self)
            except Exception, e:
                print e
                value = u''
        else:
            value = unicode(lookupattr(row, self.display_field))
        return value



from django.conf import settings
from django.conf.urls.defaults import patterns, include, url

urlpatterns = patterns('')

if 'django.contrib.admin' in settings.INSTALLED_APPS:
    from django.contrib import admin
    admin.autodiscover()
    urlpatterns += patterns('',
        url(r'^admin/', include(admin.site.urls)),
    )

if 'test_app' in settings.INSTALLED_APPS:
    urlpatterns += patterns('',
        url(r'', include('test_app.urls')),
    )

if 'django.contrib.staticfiles' in settings.INSTALLED_APPS and settings.DEBUG:
    from django.contrib.staticfiles.urls import staticfiles_urlpatterns
    urlpatterns += staticfiles_urlpatterns()
